==> link_icons-8.x-2.0.tar.gz: link_icons/src/Plugin/Field/FieldFormatter/LinkIconsFormatter.php <==
<?php

namespace Drupal\link_icons\Plugin\Field\FieldFormatter;

/**
 * @file
 * Contains \Drupal\link_icons\Plugin\Field\FieldFormatter\LinkIconsFormatter.
 */

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
 * Plugin implementation of the 'link_icons_formatter' formatter.
 *
 * @FieldFormatter (
 *   id = "link_icons_formatter",
 *   label = @Translation("Link with service icon"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkIconsFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['text'] = [
      '#type' => 'select',
      '#title' => $this->t('Text alongside the icon'),
      '#description' => $this->t('Select what text should appear with each icon (if any)'),
      '#default_value' => $this->getSetting('text'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => [
        'title' => $this->t('Link title'),
        'URL' => $this->t('Link URL'),
        'title - URL' => $this->t('Link title - link URL'),
        'title: URL' => $this->t('Link title: link URL'),
        'URL (title)' => $this->t('link URL (Link title)'),
      ],
    ];

    $form['order'] = [
      '#type' => 'select',
      '#title' => $this->t('Order of icon and text'),
      '#description' => $this->t('Select the order of each icon and its text (if any)'),
      '#default_value' => $this->getSetting('order'),
      '#required' => TRUE,
      '#options' => [
        'first' => $this->t('Icon first'),
        'last' => $this->t('Icon last'),
      ],
    ];

    $form['size'] = [
      '#type' => 'select',
      '#title' => $this->t('Size of the icon'),
      '#description' => $this->t('Select what size each icon should be (not used/supported if icons have a background [see setting below])'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#options' => [
        '1x' => $this->t('100%'),
        'lg' => $this->t('133%'),
        '2x' => $this->t('200%'),
        '3x' => $this->t('300%'),
        '4x' => $this->t('400%'),
        '5x' => $this->t('500%'),
      ],
    ];

    $form['width'] = [
      '#type' => 'select',
      '#title' => $this->t('Width of icons'),
      '#description' => $this->t('Select whether each icon should have a fixed or variable width (helps with horizontal alignment)'),
      '#default_value' => $this->getSetting('width'),
      '#required' => TRUE,
      '#options' => [
        'fixed' => $this->t('Fixed width'),
        'variable' => $this->t('Variable width'),
      ],
    ];

    $form['coloured'] = [
      '#type' => 'select',
      '#title' => $this->t('Coloured icons'),
      '#description' => $this->t("Select whether each service icon should be coloured correctly (if that's possible with a single colour)"),
      '#default_value' => $this->getSetting('coloured'),
      '#required' => TRUE,
      '#options' => [
        'coloured' => $this->t('Coloured'),
        'uncoloured' => $this->t('Uncoloured'),
      ],
    ];

    $form['shaped'] = [
      '#type' => 'select',
      '#title' => $this->t('Pre-shaped icons'),
      '#description' => $this->t('Select whether each service icon should be the pre-squared or pre-circled version (if available)'),
      '#default_value' => $this->getSetting('shaped'),
      '#required' => TRUE,
      '#options' => [
        'squared' => $this->t('Squared'),
        'circled' => $this->t('Circled'),
        'squared or else circled' => $this->t('Squared, or else circled'),
        'circled or else squared' => $this->t('Circled, or else squared'),
        'natural' => $this->t('Natural'),
      ],
    ];

    $form['background'] = [
      '#type' => 'select',
      '#title' => $this->t('Background of the icons'),
      '#description' => $this->t('Select what background each icon should have (if any)'),
      '#default_value' => $this->getSetting('background'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => [
        'square' => $this->t('Square with rounded corners solid'),
        'square-o' => $this->t('Square with rounded corners outline'),
        'stop' => $this->t('Square solid'),
        'play' => $this->t('Triangle pointing right (play symbol) solid'),
        'circle'  => $this->t('Circle solid'),
        'circle-o' => $this->t('Circle outline'),
        'circle-o-notch' => $this->t('Circle outline with notch at the top'),
        'circle-thin' => $this->t('Circle outline hairline'),
        'dot-circle-o' => $this->t('Circle outline with centre dot'),
        'cloud' => $this->t('Cloud solid'),
        'sun-o' => $this->t('Sun outline'),
        'folder' => $this->t('Folder solid'),
        'folder-o' => $this->t('Folder outline'),
        'file' => $this->t('File solid'),
        'file-o' => $this->t('File outline'),
        'heart' => $this->t('Heart solid'),
        'heart-o' => $this->t('Heart outline'),
        'laptop' => $this->t('Laptop'),
        'tablet' => $this->t('Tablet'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = $this->t('Output @text text with @size size @width width @coloured @shaped icons, @background backgrounds and the icons placed @order.',
      [
        '@text' => ($this->getSetting('text') == 'none' ? 'no' : '"' . $this->getSetting('text') . '"'),
        '@order' => $this->getSetting('order'),
        '@size' => $this->getSetting('size'),
        '@width' => $this->getSetting('width'),
        '@coloured' => $this->getSetting('coloured'),
        '@shaped' => $this->getSetting('shaped'),
        '@background' => ($this->getSetting('background') == 'none' ? 'no' : 'fa-' . $this->getSetting('background')),
      ],
    );

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'text' => 'title',
      'order' => 'first',
      'size' => '1x',
      'width' => 'fixed',
      'coloured' => 'coloured',
      'shaped' => 'natural',
      'background' => 'none',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $services = \Drupal::entityTypeManager()->getStorage('link_icon_service')->loadMultiple();

    $elements = [];

    // Step through the items.
    foreach ($items as $delta => $item) {
      $parsed = parse_url($item->getUrl()->toString());
      $chunks = explode('.', $parsed['host']);
      $host3 = '';
      if (count($chunks) == 1) {
        $host = $chunks[0];
      }
      else {
        $host = $chunks[count($chunks) - 2] . '.' . $chunks[count($chunks) - 1];
        if (count($chunks) > 2) {
          $host3 = $chunks[count($chunks) - 3] . '.' . $host;
        }
      }

      // Find the service from $services, or else make it a generic navy globe.
      unset($service);
      foreach ($services as $aservice) {
        foreach ($aservice->hostnames as $hostname) {
          if ((isset($host3) && $hostname == $host3) || $hostname == $host) {
            $service = [
              'class' => $aservice->class,
              'icon' => $aservice->icon,
              'icon_style' => $aservice->icon_style,
              'icon_square' => $aservice->icon_square ?: '',
              'icon_circle' => $aservice->icon_circle ?: '',
              'color' => $aservice->color ?: '',
            ];
          }
        }
      }

      if (!isset($service)) {
        $service = [
          'class' => 'generic',
          'icon' => 'globe',
          'icon_style' => 'solid',
          'color' => 'navy',
        ];
      }

      $hollow_endings = [
        '-o',
        '-o-notch',
        '-thin',
        'sun',
        'laptop',
        'tablet',
      ];

      // Determine if the icon needs to be inverted.
      if ($this->getSetting('background') != 'none') {
        $inverse = TRUE;
        foreach ($hollow_endings as $ending) {
          if ($inverse) {
            $ending_length = strlen($ending);
            if ($ending_length <= strlen($this->getSetting('background'))) {
              if (substr_compare($this->getSetting('background'), $ending, -$ending_length) === 0) {
                $inverse = FALSE;
              }
            }
          }
        }
      }

      switch ($this->getSetting('shaped')) {
        case 'squared':
          $icon = $service['icon_square'] ?: $service['icon'];
          break;

        case 'squared or else circled':
          $icon = $service['icon_square'] ?: ($service['icon_circle'] ?: $service['icon']);
          break;

        case 'circled':
          $icon = $service['icon_circle'] ?: $service['icon'];
          break;

        case 'circled or else squared':
          $icon = $service['icon_circle'] ?: ($service['icon_square'] ?: $service['icon']);
          break;

        case 'natural':
        default:
          $icon = $service['icon'];
          break;
      }

      switch ($service['icon_style']) {
        case 'solid':
          $style = 's';
          break;

        case 'regular':
          $style = 'r';
          break;

        case 'light':
          $style = 'l';
          break;

        case 'duotone':
          $style = 'd';
          break;

        case 'brand':
        default:
          $style = 'b';
          break;
      }

      if ($service['class'] == '') {
        $service['class'] = $service['id'];
      }

      // Generate the icon link HTML.
      $icon_link = Link::fromTextAndUrl(
        Markup::create(
          ($this->getSetting('background') != 'none' ? '<span class="fa-stack'
              . ($this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
              . '"><i class="fa fa-'
              . $this->getSetting('background')
              . ' fa-stack-2x"></i>'
            : ''
          )
          . '<i class="service ' . $service['class']
          . ' fa' . $style
          . ' fa-' . $icon
          . ($this->getSetting('background') == 'none' && $this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
          . ($this->getSetting('width') == 'fixed' ? ' fa-fw' : '')
          . ($this->getSetting('background') != 'none' ? ' fa-stack-1x' . (isset($inverse) && $inverse ? ' fa-inverse' : '') : '')
          . '"'
          . ($this->getSetting('coloured') != 'uncoloured' ? ' style="color: ' . $service['color'] . ';"' : '')
          . '></i>'
          . ($this->getSetting('background') != 'none' ? '</span>' : '')
        ),
        Url::fromUri(
          $item->getUrl()->toString(),
          [
            'attributes' => [
              'title' => $item->title,
              'target' => '_blank',
            ],
          ]
        )
      )->toString();

      // Generate the text HTML, if required, and combine it with the link icon
      // HTML in $markup.
      $text = '';
      if ($this->getSetting('text') != 'none') {
        switch ($this->getSetting('text')) {
          case 'title':
            $text = $item->title;
            break;

          case 'URL':
            $text = $item->getUrl()->toString();
            break;

          case 'title - URL':
            $text = $item->title . ' - ' . $item->getUrl()->toString();
            break;

          case 'title: URL':
            $text = $item->title . ': ' . $item->getUrl()->toString();
            break;

          case 'URL (title)':
            $text = $item->getUrl()->toString() . ' (' . $item->title . ')';
            break;
        }

        $text_link = Link::fromTextAndUrl(
          $text,
          Url::fromUri(
            $item->getUrl()->toString(),
            [
              'attributes' => [
                'title' => $item->title,
                'target' => '_blank',
              ],
            ]
          )
        )->toString();

        if ($this->getSetting('order') == 'first') {
          $markup = $icon_link . ' ' . $text_link;
        }
        else {
          $markup = $text_link . ' ' . $icon_link;
        }
      }
      else {
        $markup = $icon_link;
      }

      // Return the $markup.
      $elements[$delta] = [
        '#markup' => Markup::create($markup),
      ];
    }

    return $elements;
  }

}

==> link_icons-8.x-2.1.tar.gz: link_icons/src/Plugin/Field/FieldFormatter/LinkIconsFormatter.php <==
<?php

namespace Drupal\link_icons\Plugin\Field\FieldFormatter;

/**
 * @file
 * Contains \Drupal\link_icons\Plugin\Field\FieldFormatter\LinkIconsFormatter.
 */

use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;

/**
 * Plugin implementation of the 'link_icons_formatter' formatter.
 *
 * @FieldFormatter (
 *   id = "link_icons_formatter",
 *   label = @Translation("Link with service icon"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkIconsFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['text'] = [
      '#type' => 'select',
      '#title' => $this->t('Text alongside the icon'),
      '#description' => $this->t('Select what text should appear with each icon (if any)'),
      '#default_value' => $this->getSetting('text'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => [
        'title' => $this->t('Link title'),
        'URL' => $this->t('Link URL'),
        'title - URL' => $this->t('Link title - link URL'),
        'title: URL' => $this->t('Link title: link URL'),
        'URL (title)' => $this->t('link URL (Link title)'),
      ],
    ];

    $form['order'] = [
      '#type' => 'select',
      '#title' => $this->t('Order of icon and text'),
      '#description' => $this->t('Select the order of each icon and its text (if any)'),
      '#default_value' => $this->getSetting('order'),
      '#required' => TRUE,
      '#options' => [
        'first' => $this->t('Icon first'),
        'last' => $this->t('Icon last'),
      ],
    ];

    $form['size'] = [
      '#type' => 'select',
      '#title' => $this->t('Size of the icon'),
      '#description' => $this->t('Select what size each icon should be (not used/supported if icons have a background [see setting below])'),
      '#default_value' => $this->getSetting('size'),
      '#required' => TRUE,
      '#options' => [
        '1x' => $this->t('100%'),
        'lg' => $this->t('133%'),
        '2x' => $this->t('200%'),
        '3x' => $this->t('300%'),
        '4x' => $this->t('400%'),
        '5x' => $this->t('500%'),
      ],
    ];

    $form['width'] = [
      '#type' => 'select',
      '#title' => $this->t('Width of icons'),
      '#description' => $this->t('Select whether each icon should have a fixed or variable width (helps with horizontal alignment)'),
      '#default_value' => $this->getSetting('width'),
      '#required' => TRUE,
      '#options' => [
        'fixed' => $this->t('Fixed width'),
        'variable' => $this->t('Variable width'),
      ],
    ];

    $form['coloured'] = [
      '#type' => 'select',
      '#title' => $this->t('Coloured icons'),
      '#description' => $this->t("Select whether each service icon should be coloured correctly (if that's possible with a single colour)"),
      '#default_value' => $this->getSetting('coloured'),
      '#required' => TRUE,
      '#options' => [
        'coloured' => $this->t('Coloured'),
        'uncoloured' => $this->t('Uncoloured'),
      ],
    ];

    $form['shaped'] = [
      '#type' => 'select',
      '#title' => $this->t('Pre-shaped icons'),
      '#description' => $this->t('Select whether each service icon should be the pre-squared or pre-circled version (if available)'),
      '#default_value' => $this->getSetting('shaped'),
      '#required' => TRUE,
      '#options' => [
        'squared' => $this->t('Squared'),
        'circled' => $this->t('Circled'),
        'squared or else circled' => $this->t('Squared, or else circled'),
        'circled or else squared' => $this->t('Circled, or else squared'),
        'natural' => $this->t('Natural'),
      ],
    ];

    $form['background'] = [
      '#type' => 'select',
      '#title' => $this->t('Background of the icons'),
      '#description' => $this->t('Select what background each icon should have (if any)'),
      '#default_value' => $this->getSetting('background'),
      '#required' => FALSE,
      '#empty_value' => 'none',
      '#options' => [
        'square' => $this->t('Square with rounded corners solid'),
        'square-o' => $this->t('Square with rounded corners outline'),
        'stop' => $this->t('Square solid'),
        'play' => $this->t('Triangle pointing right (play symbol) solid'),
        'circle'  => $this->t('Circle solid'),
        'circle-o' => $this->t('Circle outline'),
        'circle-o-notch' => $this->t('Circle outline with notch at the top'),
        'circle-thin' => $this->t('Circle outline hairline'),
        'dot-circle-o' => $this->t('Circle outline with centre dot'),
        'cloud' => $this->t('Cloud solid'),
        'sun-o' => $this->t('Sun outline'),
        'folder' => $this->t('Folder solid'),
        'folder-o' => $this->t('Folder outline'),
        'file' => $this->t('File solid'),
        'file-o' => $this->t('File outline'),
        'heart' => $this->t('Heart solid'),
        'heart-o' => $this->t('Heart outline'),
        'laptop' => $this->t('Laptop'),
        'tablet' => $this->t('Tablet'),
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];

    $summary[] = $this->t('Output @text text with @size size @width width @coloured @shaped icons, @background backgrounds and the icons placed @order.',
      [
        '@text' => ($this->getSetting('text') == 'none' ? 'no' : '"' . $this->getSetting('text') . '"'),
        '@order' => $this->getSetting('order'),
        '@size' => $this->getSetting('size'),
        '@width' => $this->getSetting('width'),
        '@coloured' => $this->getSetting('coloured'),
        '@shaped' => $this->getSetting('shaped'),
        '@background' => ($this->getSetting('background') == 'none' ? 'no' : 'fa-' . $this->getSetting('background')),
      ]
    );

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'text' => 'title',
      'order' => 'first',
      'size' => '1x',
      'width' => 'fixed',
      'coloured' => 'coloured',
      'shaped' => 'natural',
      'background' => 'none',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $services = \Drupal::entityTypeManager()->getStorage('link_icon_service')->loadMultiple();

    $elements = [];

    // Step through the items.
    foreach ($items as $delta => $item) {
      $parsed = parse_url($item->getUrl()->toString());
      $chunks = explode('.', $parsed['host']);
      $host3 = '';
      if (count($chunks) == 1) {
        $host = $chunks[0];
      }
      else {
        $host = $chunks[count($chunks) - 2] . '.' . $chunks[count($chunks) - 1];
        if (count($chunks) > 2) {
          $host3 = $chunks[count($chunks) - 3] . '.' . $host;
        }
      }

      // Set the default service to a generic navy globe.
      $service = [
        'class' => 'generic',
        'icon' => 'globe',
        'icon_style' => 'solid',
        'color' => 'navy',
      ];

      // Find the service from $services.
      foreach ($services as $aservice) {
        foreach ($aservice->hostnames as $hostname) {
          if ((isset($host3) && $hostname == $host3) || $hostname == $host) {
            $service = [
              'id' => $aservice->get('id'),
              'class' => $aservice->class ?: $aservice->get('id'),
              'icon' => $aservice->icon,
              'icon_style' => $aservice->icon_style,
              'icon_square' => $aservice->icon_square ?: '',
              'icon_circle' => $aservice->icon_circle ?: '',
              'color' => $aservice->color ?: '',
            ];
          }
        }
      }

      $hollow_endings = [
        '-o',
        '-o-notch',
        '-thin',
        'sun',
        'laptop',
        'tablet',
      ];

      // Determine if the icon needs to be inverted.
      if ($this->getSetting('background') != 'none') {
        $inverse = TRUE;
        foreach ($hollow_endings as $ending) {
          if ($inverse) {
            $ending_length = strlen($ending);
            if ($ending_length <= strlen($this->getSetting('background'))) {
              if (substr_compare($this->getSetting('background'), $ending, -$ending_length) === 0) {
                $inverse = FALSE;
              }
            }
          }
        }
      }

      switch ($this->getSetting('shaped')) {
        case 'squared':
          $icon = $service['icon_square'] ?: $service['icon'];
          break;

        case 'squared or else circled':
          $icon = $service['icon_square'] ?: ($service['icon_circle'] ?: $service['icon']);
          break;

        case 'circled':
          $icon = $service['icon_circle'] ?: $service['icon'];
          break;

        case 'circled or else squared':
          $icon = $service['icon_circle'] ?: ($service['icon_square'] ?: $service['icon']);
          break;

        case 'natural':
        default:
          $icon = $service['icon'];
          break;
      }

      switch ($service['icon_style']) {
        case 'solid':
          $style = 's';
          break;

        case 'regular':
          $style = 'r';
          break;

        case 'light':
          $style = 'l';
          break;

        case 'duotone':
          $style = 'd';
          break;

        case 'brand':
        default:
          $style = 'b';
          break;
      }

      // Generate the icon link HTML.
      $icon_link = Link::fromTextAndUrl(
        Markup::create(
          ($this->getSetting('background') != 'none' ? '<span class="fa-stack'
              . ($this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
              . '"><i class="fa fa-'
              . $this->getSetting('background')
              . ' fa-stack-2x"></i>'
            : ''
          )
          . '<i class="service ' . $service['class']
          . ' fa' . $style
          . ' fa-' . $icon
          . ($this->getSetting('background') == 'none' && $this->getSetting('size') != '1x' ? ' fa-' . $this->getSetting('size') : '')
          . ($this->getSetting('width') == 'fixed' ? ' fa-fw' : '')
          . ($this->getSetting('background') != 'none' ? ' fa-stack-1x' . (isset($inverse) && $inverse ? ' fa-inverse' : '') : '')
          . '"'
          . ($this->getSetting('coloured') != 'uncoloured' ? ' style="color: ' . $service['color'] . ';"' : '')
          . '></i>'
          . ($this->getSetting('background') != 'none' ? '</span>' : '')
        ),
        Url::fromUri(
          $item->getUrl()->toString(),
          [
            'attributes' => [
              'title' => $item->title,
              'target' => '_blank',
            ],
          ]
        )
      )->toString();

      // Generate the text HTML, if required, and combine it with the link icon
      // HTML in $markup.
      $text = '';
      if ($this->getSetting('text') != 'none') {
        switch ($this->getSetting('text')) {
          case 'title':
            $text = $item->title;
            break;

          case 'URL':
            $text = $item->getUrl()->toString();
            break;

          case 'title - URL':
            $text = $item->title . ' - ' . $item->getUrl()->toString();
            break;

          case 'title: URL':
            $text = $item->title . ': ' . $item->getUrl()->toString();
            break;

          case 'URL (title)':
            $text = $item->getUrl()->toString() . ' (' . $item->title . ')';
            break;
        }

        $text_link = Link::fromTextAndUrl(
          $text,
          Url::fromUri(
            $item->getUrl()->toString(),
            [
              'attributes' => [
                'title' => $item->title,
                'target' => '_blank',
              ],
            ]
          )
        )->toString();

        if ($this->getSetting('order') == 'first') {
          $markup = $icon_link . ' ' . $text_link;
        }
        else {
          $markup = $text_link . ' ' . $icon_link;
        }
      }
      else {
        $markup = $icon_link;
      }

      // Return the $markup.
      $elements[$delta] = [
        '#markup' => Markup::create($markup),
      ];
    }

    return $elements;
  }

}
